@@ -1,8 +1,9 @@
 /**
  * @file gntpounce.c GNT Buddy Pounce API
  * @ingroup finch
- *
- * finch
+ */
+
+/* finch
  *
  * Finch is the legal property of its developers, whose names are too numerous
  * to list here.  Please refer to the COPYRIGHT file distributed with this
@@ -20,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  *
  */
 #include <gnt.h>
@@ -32,6 +33,8 @@
 #include <gntlabel.h>
 #include <gntline.h>
 #include <gnttree.h>
+#include <gntutils.h>
+
 #include "internal.h"
 #include "finch.h"
 
@@ -137,12 +140,12 @@ add_pounce_to_treeview(GntTree *tree, PurplePounce *pounce)
 static void
 populate_pounces_list(PouncesManager *dialog)
 {
-	const GList *pounces;
+	GList *pounces;
 
 	gnt_tree_remove_all(GNT_TREE(dialog->tree));
 
-	for (pounces = purple_pounces_get_all(); pounces != NULL;
-			pounces = g_list_next(pounces))
+	for (pounces = purple_pounces_get_all_for_ui(FINCH_UI); pounces != NULL;
+			pounces = g_list_delete_link(pounces, pounces))
 	{
 		add_pounce_to_treeview(GNT_TREE(dialog->tree), pounces->data);
 	}
@@ -164,6 +167,17 @@ signed_on_off_cb(PurpleConnection *gc, gpointer user_data)
 	update_pounces();
 }
 
+static void
+setup_buddy_list_suggestion(GntEntry *entry, gboolean offline)
+{
+	PurpleBlistNode *node = purple_blist_get_root();
+	for (; node; node = purple_blist_node_next(node, offline)) {
+		if (!PURPLE_BLIST_NODE_IS_BUDDY(node))
+			continue;
+		gnt_entry_add_suggest(entry, purple_buddy_get_name((PurpleBuddy*)node));
+	}
+}
+
 static void
 save_pounce_cb(GntWidget *w, PurpleGntPounceDialog *dialog)
 {
@@ -357,6 +371,8 @@ finch_pounce_editor_show(PurpleAccount *account, const char *name,
 	dialog->buddy_entry = gnt_entry_new(NULL);
 	gnt_box_add_widget(GNT_BOX(hbox), dialog->buddy_entry);
 
+	setup_buddy_list_suggestion(GNT_ENTRY(dialog->buddy_entry), TRUE);
+
 	gnt_box_add_widget(GNT_BOX(window), hbox);
 
 	if (cur_pounce != NULL) {
@@ -450,7 +466,7 @@ finch_pounce_editor_show(PurpleAccount *account, const char *name,
 
 	gnt_box_add_widget(GNT_BOX(window), gnt_line_new(FALSE));
 	/* Now the button box! */
-	bbox = gnt_hbox_new(TRUE);
+	bbox = gnt_hbox_new(FALSE);
 
 	/* Cancel button */
 	button = gnt_button_new(_("Cancel"));
@@ -611,6 +627,12 @@ pounces_manager_destroy_cb(GntWidget *widget, gpointer user_data)
 static void
 pounces_manager_add_cb(GntButton *button, gpointer user_data)
 {
+	if (purple_accounts_get_all() == NULL) {
+		purple_notify_error(NULL, _("Cannot create pounce"),
+				_("You do not have any accounts."),
+				_("You must create an account first before you can create a pounce."));
+		return;
+	}
 	finch_pounce_editor_show(NULL, NULL, NULL);
 }
 
@@ -620,7 +642,8 @@ pounces_manager_modify_cb(GntButton *button, gpointer user_data)
 {
 	PouncesManager *dialog = user_data;
 	PurplePounce *pounce = gnt_tree_get_selection_data(GNT_TREE(dialog->tree));
-	finch_pounce_editor_show(NULL, NULL, pounce);
+	if (pounce)
+		finch_pounce_editor_show(NULL, NULL, pounce);
 }
 
 static void
@@ -643,6 +666,9 @@ pounces_manager_delete_cb(GntButton *button, gpointer user_data)
 	char *buf;
 
 	pounce = (PurplePounce *)gnt_tree_get_selection_data(GNT_TREE(dialog->tree));
+	if (pounce == NULL)
+		return;
+
 	account = purple_pounce_get_pouncer(pounce);
 	pouncer = purple_account_get_username(account);
 	pouncee = purple_pounce_get_pouncee(pounce);
@@ -672,6 +698,7 @@ finch_pounces_manager_show(void)
 	GntWidget *win;
 
 	if (pounces_manager != NULL) {
+		gnt_window_present(pounces_manager->window);
 		return;
 	}
 
@@ -693,11 +720,12 @@ finch_pounces_manager_show(void)
 	gnt_box_add_widget(GNT_BOX(win), tree);
 
 	/* Button box. */
-	bbox = gnt_hbox_new(TRUE);
+	bbox = gnt_hbox_new(FALSE);
 
 	/* Add button */
 	button = gnt_button_new(_("Add"));
 	gnt_box_add_widget(GNT_BOX(bbox), button);
+	gnt_util_set_trigger_widget(tree, GNT_KEY_INS, button);
 
 	g_signal_connect(G_OBJECT(button), "activate",
 					 G_CALLBACK(pounces_manager_add_cb), dialog);
@@ -714,6 +742,7 @@ finch_pounces_manager_show(void)
 	button = gnt_button_new(_("Delete"));
 	dialog->delete_button = button;
 	gnt_box_add_widget(GNT_BOX(bbox), button);
+	gnt_util_set_trigger_widget(tree, GNT_KEY_DEL, button);
 
 	g_signal_connect(G_OBJECT(button), "activate",
 					 G_CALLBACK(pounces_manager_delete_cb), dialog);
@@ -948,10 +977,15 @@ finch_pounces_init(void)
 						PURPLE_CALLBACK(signed_on_off_cb), NULL);
 }
 
+static void
+dummy_pounce_cb(PurplePounce *pounce, PurplePounceEvent events, void *data)
+{
+}
+
 /* XXX: There's no such thing in pidgin. Perhaps there should be? */
 void finch_pounces_uninit()
 {
-	purple_pounces_register_handler(FINCH_UI, NULL, NULL, NULL);
+	purple_pounces_register_handler(FINCH_UI, dummy_pounce_cb, NULL, NULL);
 
 	purple_signals_disconnect_by_handle(finch_pounces_get_handle());
 }
